@@ -35,7 +35,6 @@ const Zpj = imports.gi.Zpj;
 const _ = imports.gettext.gettext;
 
 const Lang = imports.lang;
-const Mainloop = imports.mainloop;
 const Signals = imports.signals;
 
 const Application = imports.application;
@@ -65,7 +64,7 @@ const DeleteItemJob = new Lang.Class({
                 try {
                     object.update_finish(res);
                 } catch (e) {
-                    log(e);
+                    logError(e, 'Failed to delete resource ' + this._urn);
                 }
 
                 if (this._callback)
@@ -101,7 +100,7 @@ const CollectionIconWatcher = new Lang.Class({
                 try {
                     cursor = object.query_finish(res);
                 } catch (e) {
-                    log('Unable to query collection items ' + e.toString());
+                    logError(e, 'Unable to query collection items');
                     return;
                 }
 
@@ -115,7 +114,7 @@ const CollectionIconWatcher = new Lang.Class({
         try {
             valid = cursor.next_finish(res);
         } catch (e) {
-            log('Unable to query collection items ' + e.toString());
+            logError(e, 'Unable to query collection items');
             cursor.close();
             return;
         }
@@ -195,8 +194,13 @@ const CollectionIconWatcher = new Lang.Class({
 
         this._docs.forEach(
             function(doc) {
-                if (doc.origPixbuf)
+                if (doc.origPixbuf) {
+                    if (doc._thumbPath && !doc._failedThumbnailing)
+                        doc.origPixbuf.set_option('-documents-has-thumb', 'true');
+                    else
+                        doc.origPixbuf.remove_option('-documents-has-thumb');
                     pixbufs.push(doc.origPixbuf);
+                }
             });
 
         this._pixbuf = GdPrivate.create_collection_icon(
@@ -229,8 +233,6 @@ const DocCommon = new Lang.Class({
     _init: function(cursor) {
         this.id = null;
         this.uri = null;
-        this.uriToLoad = null;
-        this.filename = null;
         this.name = null;
         this.author = null;
         this.mtime = null;
@@ -308,12 +310,12 @@ const DocCommon = new Lang.Class({
             this.uri = '';
 
         let title = cursor.get_string(Query.QueryColumns.TITLE)[0];
-        this.filename = cursor.get_string(Query.QueryColumns.FILENAME)[0];
+        let filename = cursor.get_string(Query.QueryColumns.FILENAME)[0];
 
         if (title && title != '')
             this.name = title;
-        else if (this.filename)
-            this.name = GdPrivate.filename_strip_extension(this.filename);
+        else if (filename)
+            this.name = GdPrivate.filename_strip_extension(filename);
         else
             this.name = '';
 
@@ -342,7 +344,7 @@ const DocCommon = new Lang.Class({
                 pixbuf = iconInfo.load_icon();
                 this._setOrigPixbuf(pixbuf);
             } catch (e) {
-                log('Unable to load pixbuf: ' + e.toString());
+                logError(e, 'Unable to load pixbuf');
             }
         }
     },
@@ -360,101 +362,6 @@ const DocCommon = new Lang.Class({
         }
     },
 
-    download: function(useCache, cancellable, callback) {
-        let localFile = Gio.File.new_for_uri(this.uriToLoad);
-        let localPath = localFile.get_path();
-        let localDir = GLib.path_get_dirname(localPath);
-        GLib.mkdir_with_parents(localDir, 448);
-
-        if (!useCache) {
-            Utils.debug('Downloading ' + this.__name__ + ' ' + this.id + ' to ' + this.uriToLoad +
-                        ': bypass cache ');
-            this.downloadImpl(localFile, cancellable, callback);
-            return;
-        }
-
-        localFile.query_info_async(Gio.FILE_ATTRIBUTE_TIME_MODIFIED,
-                                   Gio.FileQueryInfoFlags.NONE,
-                                   GLib.PRIORITY_DEFAULT,
-                                   cancellable,
-                                   Lang.bind(this,
-            function(object, res) {
-                let info;
-
-                try {
-                    info = object.query_info_finish(res);
-                } catch (e) {
-                    Utils.debug('Downloading ' + this.__name__ + ' ' + this.id + ' to ' + this.uriToLoad +
-                                ': cache miss');
-                    this.downloadImpl(localFile, cancellable, callback);
-                    return;
-                }
-
-                let cacheMtime = info.get_attribute_uint64(Gio.FILE_ATTRIBUTE_TIME_MODIFIED);
-                if (this.mtime <= cacheMtime) {
-                    callback(true, null);
-                    return;
-                }
-
-                Utils.debug('Downloading ' + this.__name__ + ' ' + this.id + ' to ' + this.uriToLoad +
-                            ': cache stale (' + this.mtime + ' > ' + cacheMtime + ')');
-                this.downloadImpl(localFile, cancellable, callback);
-            }));
-    },
-
-    downloadImpl: function(localFile, cancellable, callback) {
-        throw(new Error('DocCommon implementations must override downloadImpl'));
-    },
-
-    load: function(passwd, cancellable, callback) {
-        Utils.debug('Loading ' + this.__name__ + ' ' + this.id);
-
-        if (this.collection) {
-            Mainloop.idle_add(Lang.bind(this,
-                function() {
-                    let error = new GLib.Error(Gio.IOErrorEnum,
-                                               Gio.IOErrorEnum.NOT_SUPPORTED,
-                                               "Collections can't be loaded");
-                    callback(this, null, error);
-                    return GLib.SOURCE_REMOVE;
-                }));
-
-            return;
-        }
-
-        this.download(true, cancellable, Lang.bind(this,
-            function(fromCache, error) {
-                if (error) {
-                    callback(this, null, error);
-                    return;
-                }
-
-                this.loadLocal(passwd, cancellable, Lang.bind(this,
-                    function(doc, docModel, error) {
-                        if (error) {
-                            if (fromCache &&
-                                !error.matches(EvDocument.DocumentError, EvDocument.DocumentError.ENCRYPTED)) {
-                                this.download(false, cancellable, Lang.bind(this,
-                                    function(fromCache, error) {
-                                        if (error) {
-                                            callback(this, null, error);
-                                            return;
-                                        }
-
-                                        this.loadLocal(passwd, cancellable, callback);
-                                    }));
-                            } else {
-                                callback(this, null, error);
-                            }
-
-                            return;
-                        }
-
-                        callback(this, docModel, null);
-                    }));
-            }));
-    },
-
     canEdit: function() {
         log('Error: DocCommon implementations must override canEdit');
     },
@@ -468,9 +375,6 @@ const DocCommon = new Lang.Class({
     },
 
     canPrint: function(docModel) {
-        if (this.collection)
-            return false;
-
         if (!docModel)
             return false;
 
@@ -524,7 +428,7 @@ const DocCommon = new Lang.Class({
         try {
             info = object.query_info_finish(res);
         } catch (e) {
-            log('Unable to query info for file at ' + this.uri + ': ' + e.toString());
+            logError(e, 'Unable to query info for file at ' + this.uri);
             this._failedThumbnailing = true;
             return;
         }
@@ -555,7 +459,7 @@ const DocCommon = new Lang.Class({
         try {
             info = object.query_info_finish(res);
         } catch (e) {
-            log('Unable to query info for file at ' + this.uri + ': ' + e.toString());
+            logError(e, 'Unable to query info for file at ' + this.uri);
             this._failedThumbnailing = true;
             return;
         }
@@ -583,7 +487,9 @@ const DocCommon = new Lang.Class({
                                     let pixbuf = GdkPixbuf.Pixbuf.new_from_stream_finish(res);
                                     this._setOrigPixbuf(pixbuf);
                                 } catch (e) {
-                                    log('Unable to create pixbuf from ' + thumbFile.get_uri() + ': ' + e.toString());
+                                    if (!e.matches(GdkPixbuf.PixbufError, GdkPixbuf.PixbufError.UNKNOWN_TYPE))
+                                        logError(e, 'Unable to create pixbuf from ' + thumbFile.get_uri());
+
                                     this._failedThumbnailing = true;
                                     this._thumbPath = null;
                                     thumbFile.delete_async(GLib.PRIORITY_DEFAULT, null, null);
@@ -593,7 +499,7 @@ const DocCommon = new Lang.Class({
                                 stream.close_async(0, null, null);
                             }));
                 } catch (e) {
-                    log('Unable to read file at ' + thumbFile.get_uri() + ': ' + e.toString());
+                    logError(e, 'Unable to read file at ' + thumbFile.get_uri());
                     this._failedThumbnailing = true;
                     this._thumbPath = null;
                     thumbFile.delete_async(GLib.PRIORITY_DEFAULT, null, null);
@@ -655,7 +561,7 @@ const DocCommon = new Lang.Class({
 
                 emblemedPixbuf = iconInfo.load_icon();
             } catch (e) {
-                log('Unable to render the emblem: ' + e.toString());
+                logError(e, 'Unable to render the emblem');
             }
         }
 
@@ -687,8 +593,6 @@ const DocCommon = new Lang.Class({
     },
 
     loadLocal: function(passwd, cancellable, callback) {
-        Utils.debug('Loading ' + this.__name__ + ' ' + this.id + ' from ' + this.uriToLoad);
-
         if (this.mimeType == 'application/x-mobipocket-ebook' ||
             this.mimeType == 'application/x-fictionbook+xml' ||
             this.mimeType == 'application/x-zip-compressed-fb2') {
@@ -715,7 +619,7 @@ const DocCommon = new Lang.Class({
             return;
         }
 
-        GdPrivate.pdf_loader_load_uri_async(this.uriToLoad, passwd, cancellable, Lang.bind(this,
+        GdPrivate.pdf_loader_load_uri_async(this.uri, passwd, cancellable, Lang.bind(this,
             function(source, res) {
                 try {
                     let docModel = GdPrivate.pdf_loader_load_uri_finish(res);
@@ -738,7 +642,7 @@ const DocCommon = new Lang.Class({
             else
                 Gtk.show_uri(screen, this.uri, timestamp);
         } catch (e) {
-            log('Unable to show URI ' + this.uri + ': ' + e.toString());
+            logError(e, 'Unable to show URI ' + this.uri);
         }
     },
 
@@ -746,7 +650,7 @@ const DocCommon = new Lang.Class({
         this.load(null, null, Lang.bind(this,
             function(doc, docModel, error) {
                 if (error) {
-                    log('Unable to print document ' + this.uri + ': ' + error);
+                    logError(error, 'Unable to print document ' + this.uri);
                     return;
                 }
 
@@ -755,11 +659,6 @@ const DocCommon = new Lang.Class({
 
                 let printOp = EvView.PrintOperation.new(docModel.get_document());
 
-                printOp.connect('begin-print', Lang.bind(this,
-                    function() {
-                        Application.selectionController.setSelectionMode(false);
-                    }));
-
                 printOp.connect('done', Lang.bind(this,
                     function(op, res) {
                         if (res == Gtk.PrintOperationResult.ERROR) {
@@ -846,7 +745,6 @@ const LocalDocument = new Lang.Class({
 
     populateFromCursor: function(cursor) {
         this.parent(cursor);
-        this.uriToLoad = this.uri;
 
         if (!Application.application.gettingStartedLocation)
             return;
@@ -881,7 +779,6 @@ const LocalDocument = new Lang.Class({
     },
 
     load: function(passwd, cancellable, callback) {
-        Utils.debug('Loading ' + this.__name__ + ' ' + this.id);
         this.loadLocal(passwd, cancellable, callback);
     },
 
@@ -907,7 +804,7 @@ const LocalDocument = new Lang.Class({
                 try {
                     file.trash_finish(res);
                 } catch(e) {
-                    log('Unable to trash ' + this.uri + ': ' + e.message);
+                    logError(e, 'Unable to trash ' + this.uri);
                 }
             }));
     },
@@ -966,59 +863,36 @@ const GoogleDocument = new Lang.Class({
                  }));
     },
 
-    downloadImpl: function(localFile, cancellable, callback) {
+    load: function(passwd, cancellable, callback) {
         this.createGDataEntry(cancellable, Lang.bind(this,
-            function(entry, service, error) {
-                if (error) {
-                    callback(false, error);
+            function(entry, service, exception) {
+                if (exception) {
+                    // try loading from the most recent cache, if any
+                    GdPrivate.pdf_loader_load_uri_async(this.identifier, passwd, cancellable, Lang.bind(this,
+                        function(source, res) {
+                            try {
+                                let docModel = GdPrivate.pdf_loader_load_uri_finish(res);
+                                callback(this, docModel, null);
+                            } catch (e) {
+                                // report the outmost error only
+                                callback(this, null, exception);
+                            }
+                        }));
+
                     return;
                 }
 
-                Utils.debug('Created GDataEntry for ' + this.id);
-
-                let inputStream;
-
-                try {
-                    inputStream = entry.download(service, 'pdf', cancellable);
-                } catch(e) {
-                    callback(false, e);
-                    return;
-                }
-
-                localFile.replace_async(null,
-                                        false,
-                                        Gio.FileCreateFlags.PRIVATE,
-                                        GLib.PRIORITY_DEFAULT,
-                                        cancellable,
-                                        Lang.bind(this,
-                    function(object, res) {
-                        let outputStream;
-
-                        try {
-                            outputStream = object.replace_finish(res);
-                        } catch (e) {
-                            callback(false, e);
-                            return;
-                        }
-
-                        outputStream.splice_async(inputStream,
-                                                  Gio.OutputStreamSpliceFlags.CLOSE_SOURCE |
-                                                  Gio.OutputStreamSpliceFlags.CLOSE_TARGET,
-                                                  GLib.PRIORITY_DEFAULT,
-                                                  cancellable,
-                                                  Lang.bind(this,
-                            function(object, res) {
-                                try {
-                                    object.splice_finish(res);
-                                } catch (e) {
-                                    callback(false, e);
-                                    return;
-                                }
-
-                                callback(false, null);
-                            }));
-                    }));
-            }))
+                GdPrivate.pdf_loader_load_gdata_entry_async
+                    (entry, service, cancellable, Lang.bind(this,
+                        function(source, res) {
+                            try {
+                                let docModel = GdPrivate.pdf_loader_load_uri_finish(res);
+                                callback(this, docModel, null);
+                            } catch (e) {
+                                callback(this, null, e);
+                            }
+                        }));
+            }));
     },
 
     createThumbnail: function(callback) {
@@ -1098,15 +972,6 @@ const GoogleDocument = new Lang.Class({
         this.shared = cursor.get_boolean(Query.QueryColumns.SHARED);
 
         this.parent(cursor);
-
-        let localDir = GLib.build_filenamev([GLib.get_user_cache_dir(), "gnome-documents", "google"]);
-
-        let identifierHash = GLib.compute_checksum_for_string(GLib.ChecksumType.SHA1, this.identifier, -1);
-        let localFilename = identifierHash + ".pdf";
-
-        let localPath = GLib.build_filenamev([localDir, localFilename]);
-        let localFile = Gio.File.new_for_path(localPath);
-        this.uriToLoad = localFile.get_uri();
     },
 
     canEdit: function() {
@@ -1127,8 +992,6 @@ const GoogleDocument = new Lang.Class({
     }
 });
 
-const OWNCLOUD_PREFIX = 'owncloud:';
-
 const OwncloudDocument = new Lang.Class({
     Name: 'OwncloudDocument',
     Extends: DocCommon,
@@ -1148,21 +1011,6 @@ const OwncloudDocument = new Lang.Class({
             this.defaultAppName = this.defaultApp.get_name();
     },
 
-    populateFromCursor: function(cursor) {
-        this.parent(cursor);
-
-        let localDir = GLib.build_filenamev([GLib.get_user_cache_dir(), "gnome-documents", "owncloud"]);
-
-        let identifierHash = this.identifier.substring(OWNCLOUD_PREFIX.length);
-        let filenameStripped = GdPrivate.filename_strip_extension(this.filename);
-        let extension = this.filename.substring(filenameStripped.length);
-        let localFilename = identifierHash + extension;
-
-        let localPath = GLib.build_filenamev([localDir, localFilename]);
-        let localFile = Gio.File.new_for_path(localPath);
-        this.uriToLoad = localFile.get_uri();
-    },
-
     createThumbnail: function(callback) {
         GdPrivate.queue_thumbnail_job_for_file_async(this._file, Lang.bind(this,
             function(object, res) {
@@ -1182,53 +1030,8 @@ const OwncloudDocument = new Lang.Class({
         this.typeDescription = description;
     },
 
-    downloadImpl: function(localFile, cancellable, callback) {
-        let remoteFile = Gio.File.new_for_uri(this.uri);
-        remoteFile.read_async(GLib.PRIORITY_DEFAULT, cancellable, Lang.bind(this,
-            function(object, res) {
-                let inputStream;
-
-                try {
-                    inputStream = object.read_finish(res);
-                } catch (e) {
-                    callback(false, e);
-                    return;
-                }
-
-                localFile.replace_async(null,
-                                        false,
-                                        Gio.FileCreateFlags.PRIVATE,
-                                        GLib.PRIORITY_DEFAULT,
-                                        cancellable,
-                                        Lang.bind(this,
-                    function(object, res) {
-                        let outputStream;
-
-                        try {
-                            outputStream = object.replace_finish(res);
-                        } catch (e) {
-                            callback(false, e);
-                            return;
-                        }
-
-                        outputStream.splice_async(inputStream,
-                                                  Gio.OutputStreamSpliceFlags.CLOSE_SOURCE |
-                                                  Gio.OutputStreamSpliceFlags.CLOSE_TARGET,
-                                                  GLib.PRIORITY_DEFAULT,
-                                                  cancellable,
-                                                  Lang.bind(this,
-                            function(object, res) {
-                                try {
-                                    object.splice_finish(res);
-                                } catch (e) {
-                                    callback(false, e);
-                                    return;
-                                }
-
-                                callback(false, null);
-                            }));
-                    }));
-            }));
+    load: function(passwd, cancellable, callback) {
+        this.loadLocal(passwd, cancellable, callback);
     },
 
     canEdit: function() {
@@ -1266,21 +1069,6 @@ const SkydriveDocument = new Lang.Class({
         this.sourceName = _("OneDrive");
     },
 
-    populateFromCursor: function(cursor) {
-        this.parent(cursor);
-
-        let localDir = GLib.build_filenamev([GLib.get_user_cache_dir(), "gnome-documents", "skydrive"]);
-
-        let identifierHash = GLib.compute_checksum_for_string(GLib.ChecksumType.SHA1, this.identifier, -1);
-        let filenameStripped = GdPrivate.filename_strip_extension(this.filename);
-        let extension = this.filename.substring(filenameStripped.length);
-        let localFilename = identifierHash + extension;
-
-        let localPath = GLib.build_filenamev([localDir, localFilename]);
-        let localFile = Gio.File.new_for_path(localPath);
-        this.uriToLoad = localFile.get_uri();
-    },
-
     _createZpjEntry: function(cancellable, callback) {
         let source = Application.sourceManager.getItemById(this.resourceUrn);
 
@@ -1307,61 +1095,35 @@ const SkydriveDocument = new Lang.Class({
                  }));
     },
 
-    downloadImpl: function(localFile, cancellable, callback) {
+    load: function(passwd, cancellable, callback) {
         this._createZpjEntry(cancellable, Lang.bind(this,
-            function(entry, service, error) {
-                if (error) {
-                    callback(false, error);
+            function(entry, service, exception) {
+                if (exception) {
+                    // try loading from the most recent cache, if any
+                    GdPrivate.pdf_loader_load_uri_async(this.identifier, passwd, cancellable, Lang.bind(this,
+                        function(source, res) {
+                            try {
+                                let docModel = GdPrivate.pdf_loader_load_uri_finish(res);
+                                callback(this, docModel, null);
+                            } catch (e) {
+                                // report the outmost error only
+                                callback(this, null, exception);
+                            }
+                        }));
+
                     return;
                 }
 
-                Utils.debug('Created ZpjEntry for ' + this.id);
-
-                service.download_file_to_stream_async(entry, cancellable, Lang.bind(this,
-                    function(object, res) {
-                        let inputStream;
-
-                        try {
-                            inputStream = object.download_file_to_stream_finish(res);
-                        } catch (e) {
-                            callback(false, e);
-                            return;
-                        }
-
-                        localFile.replace_async(null,
-                                                false,
-                                                Gio.FileCreateFlags.PRIVATE,
-                                                GLib.PRIORITY_DEFAULT,
-                                                cancellable,
-                                                Lang.bind(this,
-                            function(object, res) {
-                                let outputStream;
-
-                                try {
-                                    outputStream = object.replace_finish(res);
-                                } catch (e) {
-                                    callback(false, e);
-                                    return;
-                                }
-
-                                outputStream.splice_async(inputStream,
-                                                          Gio.OutputStreamSpliceFlags.CLOSE_SOURCE |
-                                                          Gio.OutputStreamSpliceFlags.CLOSE_TARGET,
-                                                          GLib.PRIORITY_DEFAULT,
-                                                          cancellable,
-                                                          Lang.bind(this,
-                                    function(object, res) {
-                                        try {
-                                            object.splice_finish(res);
-                                        } catch (e) {
-                                            callback(false, e);
-                                            return;
-                                        }
-
-                                        callback(false, null);
-                                    }));
-                            }));
-                    }));
+                GdPrivate.pdf_loader_load_zpj_entry_async
+                    (entry, service, cancellable, Lang.bind(this,
+                        function(source, res) {
+                            try {
+                                let docModel = GdPrivate.pdf_loader_load_zpj_entry_finish(res);
+                                callback(this, docModel, null);
+                            } catch (e) {
+                                callback(this, null, e);
+                            }
+                        }));
             }));
     },
 
@@ -1460,7 +1222,7 @@ const DocumentManager = new Lang.Class({
 
     _identifierIsOwncloud: function(identifier) {
         return (identifier &&
-                (identifier.indexOf(OWNCLOUD_PREFIX) != -1));
+                (identifier.indexOf('owncloud:') != -1));
     },
 
     _identifierIsSkydrive: function(identifier) {
@@ -1592,8 +1354,6 @@ const DocumentManager = new Lang.Class({
             return;
         }
 
-        logError(error, 'Unable to load document');
-
         // Translators: %s is the title of a document
         let message = _("Oops! Unable to load “%s”").format(doc.name);
         let exception = this._humanizeError(error);
