@@ -22,12 +22,14 @@
 const GdPrivate = imports.gi.GdPrivate;
 const Gepub = imports.gi.Gepub;
 const Gio = imports.gi.Gio;
+const GLib = imports.gi.GLib;
 const WebKit2 = imports.gi.WebKit2;
 
 const _ = imports.gettext.gettext;
 
 const Documents = imports.documents;
 const Preview = imports.preview;
+const Utils = imports.utils;
 
 const Lang = imports.lang;
 
@@ -41,6 +43,11 @@ const EPUBView = new Lang.Class({
 
     createActions: function() {
         return [
+            { name: 'find',
+              callback: Utils.actionToggleCallback,
+              state: GLib.Variant.new('b', false),
+              stateChanged: Lang.bind(this, this._findStateChanged),
+              accels: ['<Primary>f'] },
             { name: 'find-prev',
               callback: Lang.bind(this, this.findPrev),
               accels: ['<Shift><Primary>g'] },
@@ -55,7 +62,17 @@ const EPUBView = new Lang.Class({
     },
 
     createView: function() {
-        return new Gepub.Widget();
+        let view = new Gepub.Widget();
+
+        let fc = view.get_find_controller();
+        fc.connect('found-text', Lang.bind(this, function(view, matchCount, data) {
+            let hasResults = matchCount > 0;
+
+            this.getAction('find-prev').enabled = hasResults;
+            this.getAction('find-next').enabled = hasResults;
+        }));
+
+        return view;
     },
 
     createContextMenu: function() {
@@ -65,7 +82,7 @@ const EPUBView = new Lang.Class({
     onLoadFinished: function(manager, doc) {
         this.parent(manager, doc);
 
-        let f = Gio.File.new_for_uri(doc.uriToLoad);
+        let f = Gio.File.new_for_uri(doc.uri);
         this._epubdoc = new Gepub.Doc({ path: f.get_path() });
         this._epubdoc.init(null);
 
@@ -124,8 +141,15 @@ const EPUBView = new Lang.Class({
         fc.search(str, WebKit2.FindOptions.CASE_INSENSITIVE, 0);
     },
 
-    get canFind() {
-        return true;
+    _findStateChanged: function(action) {
+        if (action.state.get_boolean()) {
+            this.toolbar.searchbar.reveal();
+        } else {
+            this.toolbar.searchbar.conceal();
+
+            let fc = this.view.get_find_controller();
+            fc.search_finish();
+        }
     },
 
     findNext: function() {
@@ -139,39 +163,13 @@ const EPUBView = new Lang.Class({
     }
 });
 
-const EPUBSearchbar = new Lang.Class({
-    Name: 'EPUBSearchbar',
-    Extends: Preview.PreviewSearchbar,
-
-    _init: function(preview) {
-        this.parent(preview);
-
-        let fc = this.preview.view.get_find_controller();
-        fc.connect('found-text', Lang.bind(this, function(view, matchCount, data) {
-            this._onSearchChanged(this.preview, matchCount > 0);
-        }));
-
-        this._onSearchChanged(this.preview, false);
-    },
-
-    _onSearchChanged: function(view, hasResults) {
-        this.preview.getAction('find-prev').enabled = hasResults;
-        this.preview.getAction('find-next').enabled = hasResults;
-    },
-
-    conceal: function() {
-        let fc = this.preview.view.get_find_controller();
-        fc.search_finish();
-
-        this.parent();
-    }
-});
-
 const EPUBViewToolbar = new Lang.Class({
     Name: 'EPUBViewToolbar',
     Extends: Preview.PreviewToolbar,
 
-    createSearchbar: function() {
-        return new EPUBSearchbar(this.preview);
+    _init: function(preview) {
+        this.parent(preview);
+
+        this.addSearchButton('view.find');
     }
 });
