@@ -32,14 +32,13 @@ const Mainloop = imports.mainloop;
 
 const Application = imports.application;
 const Documents = imports.documents;
+const FullscreenAction = imports.fullscreenAction;
 const Places = imports.places;
 const Presentation = imports.presentation;
 const Preview = imports.preview;
 const Utils = imports.utils;
 const WindowMode = imports.windowMode;
 
-const _FULLSCREEN_TOOLBAR_TIMEOUT = 2; // seconds
-
 const EvinceView = new Lang.Class({
     Name: 'EvinceView',
     Extends: Preview.Preview,
@@ -56,10 +55,10 @@ const EvinceView = new Lang.Class({
 
         this.parent(overlay, mainWindow);
 
-        Application.modeController.connect('fullscreen-changed', Lang.bind(this,
-            this._onFullscreenChanged));
-        Application.modeController.connect('window-mode-changed', Lang.bind(this,
-            this._onWindowModeChanged));
+        let fullscreenAction = new FullscreenAction.FullscreenAction({ window: mainWindow });
+        fullscreenAction.connect('notify::state', Lang.bind(this, this._onFullscreenChanged));
+        this.actionGroup.add_action(fullscreenAction);
+        Application.application.set_accels_for_action('view.fullscreen', ['F11']);
 
         this.getAction('bookmark-page').enabled = false;
 
@@ -69,6 +68,7 @@ const EvinceView = new Lang.Class({
         this.connect('destroy', Lang.bind(this,
             function() {
                 Application.application.disconnect(nightModeId);
+                fullscreenAction.change_state(new GLib.Variant('b', false));
             }));
     },
 
@@ -115,6 +115,20 @@ const EvinceView = new Lang.Class({
         }));
     },
 
+    _findStateChanged: function(action) {
+        let toolbar = this.toolbar;
+        if (this.fullscreen)
+            toolbar = this._fsToolbar.toolbar;
+
+        if (action.state.get_boolean()) {
+            toolbar.searchbar.reveal();
+            this._evView.find_set_highlight_search(true);
+        } else {
+            toolbar.searchbar.conceal();
+            this._evView.find_set_highlight_search(false);
+        }
+    },
+
     _bookmarkStateChanged: function(action) {
         let pageNumber = this._model.page;
         let bookmark = new GdPrivate.Bookmark({ page_number: pageNumber });
@@ -145,6 +159,14 @@ const EvinceView = new Lang.Class({
             doc.print(this.mainWindow);
     },
 
+    _scrollUp: function() {
+        this._evView.scroll(Gtk.ScrollType.PAGE_BACKWARD, false);
+    },
+
+    _scrollDown: function() {
+        this._evView.scroll(Gtk.ScrollType.PAGE_FORWARD, false);
+    },
+
     createActions: function() {
         let actions = [
             { name: 'zoom-in',
@@ -162,6 +184,11 @@ const EvinceView = new Lang.Class({
             { name: 'rotate-right',
               callback: Lang.bind(this, this._rotateRight),
               accels: ['<Primary>Right'] },
+            { name: 'find',
+              callback: Utils.actionToggleCallback,
+              state: GLib.Variant.new('b', false),
+              stateChanged: Lang.bind(this, this._findStateChanged),
+              accels: ['<Primary>f'] },
             { name: 'find-prev',
               callback: Lang.bind(this, this.findPrev),
               accels: ['<Shift><Primary>g'] },
@@ -180,7 +207,13 @@ const EvinceView = new Lang.Class({
               callback: Lang.bind(this, this._edit) },
             { name: 'print-current',
               callback: Lang.bind(this, this._print),
-              accels: ['<Primary>p'] }
+              accels: ['<Primary>p'] },
+            { name: 'scroll-up',
+              callback: Lang.bind(this, this._scrollUp),
+              accels: ['Page_Up'] },
+            { name: 'scroll-down',
+              callback: Lang.bind(this, this._scrollDown),
+              accels: ['Page_Down'] }
         ];
 
         if (!Application.application.isBooks)
@@ -198,8 +231,7 @@ const EvinceView = new Lang.Class({
     },
 
     createToolbar: function() {
-        this._toolbar = new EvinceViewToolbar(this);
-        return this._toolbar;
+        return new EvinceViewToolbar(this);
     },
 
     createView: function() {
@@ -235,6 +267,8 @@ const EvinceView = new Lang.Class({
         this.parent(manager, doc);
 
         this.getAction('bookmark-page').enabled = false;
+        this.getAction('find').enabled = false;
+        this.getAction('gear-menu').enabled = false;
         this.getAction('places').enabled = false;
     },
 
@@ -273,9 +307,12 @@ const EvinceView = new Lang.Class({
         this.getAction('bookmark-page').enabled = hasMultiplePages && this._bookmarks;
         this.getAction('places').enabled = hasMultiplePages;
 
+        this._enableSearch();
+        this.getAction('gear-menu').enabled = true;
+
         this._evView.set_model(this._model);
         this.navControls.setModel(this._model);
-        this._toolbar.setModel(this._model);
+        this.toolbar.setModel(this._model);
 
         this.set_visible_child_full('view', Gtk.StackTransitionType.NONE);
         this.grab_focus();
@@ -288,6 +325,23 @@ const EvinceView = new Lang.Class({
         this.parent(manager, doc, message, exception);
     },
 
+    _enableSearch: function() {
+        let canFind = true;
+
+        try {
+            // This is a hack to find out if evDoc implements the
+            // EvDocument.DocumentFind interface or not. We don't expect
+            // the following invocation to work.
+            let evDoc = this._model.get_document();
+            evDoc.find_text();
+        } catch (e if e instanceof TypeError) {
+            canFind = false;
+        } catch (e) {
+        }
+
+        this.getAction('find').enabled = (this.hasPages && canFind);
+    },
+
     _onPageChanged: function() {
         let pageNumber = this._model.page;
         this._pageChanged = true;
@@ -400,7 +454,7 @@ const EvinceView = new Lang.Class({
         try {
             Gio.AppInfo.launch_default_for_uri(uri, context);
         } catch (e) {
-            log('Unable to open external link: ' + e.message);
+            logError(e, 'Unable to open external link');
         }
     },
 
@@ -427,16 +481,11 @@ const EvinceView = new Lang.Class({
         }
     },
 
-    _onWindowModeChanged: function() {
-        let windowMode = Application.modeController.getWindowMode();
-        if (windowMode != WindowMode.WindowMode.PREVIEW_EV) {
-            this.controlsVisible = false;
-            this._hidePresentation();
-        }
-    },
+    _onFullscreenChanged: function(action) {
+        let fullscreen = action.state.get_boolean();
 
-    _onFullscreenChanged: function() {
-        let fullscreen = Application.modeController.getFullscreen();
+        this.toolbar.visible = !fullscreen;
+        this.toolbar.sensitive = !fullscreen;
 
         if (fullscreen) {
             // create fullscreen toolbar (hidden by default)
@@ -448,9 +497,14 @@ const EvinceView = new Lang.Class({
                 function() {
                     this.controlsVisible = true;
                 }));
+
+            Application.application.set_accels_for_action('view.fullscreen',
+                                                          ['F11', 'Escape']);
         } else {
             this._fsToolbar.destroy();
             this._fsToolbar = null;
+
+            Application.application.set_accels_for_action('view.fullscreen', ['F11']);
         }
 
         this._syncControlsVisible();
@@ -537,6 +591,8 @@ const EvinceView = new Lang.Class({
     },
 
     search: function(str) {
+        this._evView.find_search_changed();
+
         if (!this._model)
             return;
 
@@ -565,7 +621,10 @@ const EvinceView = new Lang.Class({
         // FIXME: ev_job_find_get_results() returns a GList **
         // and thus is not introspectable
         GdPrivate.ev_view_find_changed(this._evView, job, page);
-        this.emitJS('search-changed', job.has_results());
+
+        let hasResults = job.has_results();
+        this.getAction('find-prev').enabled = hasResults;
+        this.getAction('find-next').enabled = hasResults;
     },
 
     _loadMetadata: function() {
@@ -618,19 +677,10 @@ const EvinceView = new Lang.Class({
         return this._model ? this._model.document.get_n_pages() : 0;
     },
 
-    get canFind() {
-        return true;
-    },
-
-    scroll: function(direction) {
-        this._evView.scroll(direction, false);
-    },
-
-    get evView() {
-        return this._evView;
+    get fullscreen() {
+        return this.getAction('fullscreen').state.get_boolean();
     }
 });
-Utils.addJSSignalMethods(EvinceView.prototype);
 
 const EvinceViewNavControls = new Lang.Class({
     Name: 'EvinceViewNavControls',
@@ -675,84 +725,49 @@ const EvinceViewToolbar = new Lang.Class({
     _init: function(preview) {
         this.parent(preview);
 
+        this._fsStateId = 0;
         this._handleEvent = false;
-        this._searchAction = Application.application.lookup_action('search');
-        this._searchAction.enabled = false;
 
-        this.preview.getAction('gear-menu').enabled = false;
+        this.addSearchButton('view.find');
 
         if (Application.application.isBooks) {
-            this.addFullscreenButton();
+            this._addFullscreenButton();
             this.addNightmodeButton();
         }
 
         this.connect('destroy', Lang.bind(this, function() {
-            this._searchAction.enabled = true;
+            if (this._fsStateId > 0)
+                this.preview.getAction('fullscreen').disconnect(this._fsStateId);
         }));
     },
 
-    _enableSearch: function() {
-        let hasPages = this.preview.hasPages;
-        let canFind = true;
+    _addFullscreenButton: function() {
+        this._fullscreenButton = new Gtk.Button({ image: new Gtk.Image ({ icon_name: 'view-fullscreen-symbolic' }),
+                                                  tooltip_text: _("Fullscreen"),
+                                                  action_name: 'view.fullscreen',
+                                                  visible: true });
+        this.toolbar.pack_end(this._fullscreenButton);
 
-        try {
-            // This is a hack to find out if evDoc implements the
-            // EvDocument.DocumentFind interface or not. We don't expect
-            // the following invocation to work.
-            let evDoc = this.preview.getModel().get_document();
-            evDoc.find_text();
-        } catch (e if e instanceof TypeError) {
-            canFind = false;
-        } catch (e) {
-        }
-
-        this._handleEvent = (hasPages && canFind);
-        this._searchAction.enabled = (hasPages && canFind);
+        let action = this.preview.getAction('fullscreen');
+        this._fsStateId = action.connect('notify::state', Lang.bind(this, this._fullscreenStateChanged));
+        this._fullscreenStateChanged();
     },
 
-    createSearchbar: function() {
-        return new EvinceViewSearchbar(this.preview);
+    _fullscreenStateChanged: function() {
+        let action = this.preview.getAction('fullscreen');
+        let fullscreen = action.state.get_boolean();
+
+        if (fullscreen)
+            this._fullscreenButton.image.icon_name = 'view-restore-symbolic';
+        else
+            this._fullscreenButton.image.icon_name ='view-fullscreen-symbolic';
     },
 
     setModel: function() {
-        this.preview.getAction('gear-menu').enabled = true;
-        this._enableSearch();
         this.updateTitle();
     }
 });
 
-const EvinceViewSearchbar = new Lang.Class({
-    Name: 'EvinceViewSearchbar',
-    Extends: Preview.PreviewSearchbar,
-
-    _init: function(preview) {
-        this.parent(preview);
-
-        this.preview.connectJS('search-changed', Lang.bind(this, this._onSearchChanged));
-        this._onSearchChanged(this.preview, false);
-    },
-
-    _onSearchChanged: function(view, hasResults) {
-        this.preview.getAction('find-prev').enabled = hasResults;
-        this.preview.getAction('find-next').enabled = hasResults;
-    },
-
-    entryChanged: function() {
-        this.preview.evView.find_search_changed();
-        this.parent();
-    },
-
-    reveal: function() {
-        this.preview.evView.find_set_highlight_search(true);
-        this.parent();
-    },
-
-    conceal: function() {
-        this.preview.evView.find_set_highlight_search(false);
-        this.parent();
-    }
-});
-
 const EvinceViewFullscreenToolbar = new Lang.Class({
     Name: 'EvinceViewFullscreenToolbar',
     Extends: Gtk.Revealer,
@@ -760,43 +775,40 @@ const EvinceViewFullscreenToolbar = new Lang.Class({
     _init: function(previewView) {
         this.parent({ valign: Gtk.Align.START });
 
-        this._toolbar = new EvinceViewToolbar(previewView);
+        this.toolbar = new EvinceViewToolbar(previewView);
 
-        this.add(this._toolbar);
+        this.add(this.toolbar);
         this.show();
 
         // make controls show when a toolbar action is activated in fullscreen
-        let actionNames = ['gear-menu', 'search'];
+        let actionNames = ['gear-menu', 'find'];
         let signalIds = [];
 
-        actionNames.forEach(Lang.bind(this,
-            function(actionName) {
-                let signalName = 'action-state-changed::' + actionName;
-                let signalId = Application.application.connect(signalName, Lang.bind(this,
-                    function(actionGroup, actionName, value) {
-                        let state = value.get_boolean();
-                        if (state)
-                            this.emitJS('show-controls');
-                    }));
+        actionNames.forEach(Lang.bind(this, function(actionName) {
+            let signalName = 'action-state-changed::' + actionName;
+            let signalId = previewView.actionGroup.connect(signalName, Lang.bind(this,
+                function(actionGroup, actionName, value) {
+                    let state = value.get_boolean();
+                    if (state)
+                        this.emitJS('show-controls');
+                }));
 
-                signalIds.push(signalId);
-            }));
+            signalIds.push(signalId);
+        }));
 
-        this._toolbar.connect('destroy', Lang.bind(this,
-            function() {
-                signalIds.forEach(
-                    function(signalId) {
-                        Application.application.disconnect(signalId);
-                    });
-            }));
+        this.toolbar.connect('destroy', Lang.bind(this, function() {
+            signalIds.forEach(function(signalId) {
+                previewView.actionGroup.disconnect(signalId);
+            });
+        }));
     },
 
     handleEvent: function(event) {
-        this._toolbar.handleEvent(event);
+        this.toolbar.handleEvent(event);
     },
 
     setModel: function(model) {
-        this._toolbar.setModel(model);
+        this.toolbar.setModel(model);
     },
 
     reveal: function() {
@@ -805,7 +817,7 @@ const EvinceViewFullscreenToolbar = new Lang.Class({
 
     conceal: function() {
         this.set_reveal_child(false);
-        Application.application.change_action_state('search', GLib.Variant.new('b', false));
+        this.toolbar.preview.getAction('find').change_state(GLib.Variant.new('b', false));
     }
 });
 Utils.addJSSignalMethods(EvinceViewFullscreenToolbar.prototype);
