@@ -40,10 +40,7 @@ const Utils = imports.utils;
 const ZOOM_IN_FACTOR = 1.2;
 const ZOOM_OUT_FACTOR = (1.0/ZOOM_IN_FACTOR);
 
-const openDocumentFormats = ['application/msword',
-                             'application/vnd.ms-excel',
-                             'application/vnd.ms-powerpoint',
-                             'application/vnd.oasis.opendocument.text',
+const openDocumentFormats = ['application/vnd.oasis.opendocument.text',
                              'application/vnd.oasis.opendocument.text-template',
                              'application/vnd.oasis.opendocument.text-web',
                              'application/vnd.oasis.opendocument.text-master',
@@ -66,10 +63,9 @@ const openDocumentFormats = ['application/msword',
                              'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
                              'application/vnd.openxmlformats-officedocument.wordprocessingml.template',];
 
+
 // These are the documents consisting of document parts.
-const openDocumentPartFormats = ['application/vnd.ms-excel',
-                                 'application/vnd.ms-powerpoint',
-                                 'application/vnd.oasis.opendocument.presentation',
+const openDocumentPartFormats = ['application/vnd.oasis.opendocument.presentation',
                                  'application/vnd.oasis.opendocument.presentation-template',
                                  'application/vnd.oasis.opendocument.spreadsheet',
                                  'application/vnd.oasis.opendocument.spreadsheet-template',
@@ -128,7 +124,7 @@ const LOKView = new Lang.Class({
         sw.get_style_context().add_class('documents-scrolledwin');
 
         if (isAvailable()) {
-            this._lokview = LOKDocView.View.new(null, null);
+            this._lokview = LOKDocView.View.new(null, null, null);
             sw.add(this._lokview);
 
             this._lokview.show();
