@@ -145,7 +145,7 @@ const SearchTypeManager = new Lang.Class({
                                         where: '?urn rdf:type nfo:EBook .' }));
           this.addItem(new SearchType({ id: SearchTypeStock.COMICS,
                                         name: _("Comics"),
-                                        filter: '(nie:mimeType(?urn) IN (\"application/x-cbr\", \"application/x-cbz\", \"application/x-cbt\", \"application/x-cb7\"))',
+                                        filter: '(nie:mimeType(?urn) IN (\"application/x-cbr\", \"application/x-cbz\", \"application/vnd.comicbook+zip\", \"application/x-cbt\", \"application/x-cb7\"))',
                                         where: '?urn rdf:type nfo:EBook .' }));
         } else {
             this.addItem(new SearchType({ id: SearchTypeStock.PRESENTATIONS,
@@ -583,7 +583,7 @@ const OffsetController = new Lang.Class({
                     try {
                         cursor = object.query_finish(res);
                     } catch (e) {
-                        log('Unable to execute count query: ' + e.toString());
+                        logError(e, 'Unable to execute count query');
                         return;
                     }
 
