@@ -61,7 +61,7 @@ const FetchCollectionsJob = new Lang.Class({
                     cursor = object.query_finish(res);
                     cursor.next_async(null, Lang.bind(this, this._onCursorNext));
                 } catch (e) {
-                    log('Unable to run FetchCollectionsJob: ' + e.message);
+                    logError(e, 'Unable to run FetchCollectionsJob');
                     this._emitCallback();
                 }
             }));
@@ -73,7 +73,7 @@ const FetchCollectionsJob = new Lang.Class({
         try {
             valid = cursor.next_finish(res);
         } catch (e) {
-            log('Unable to read results of FetchCollectionsJob: ' + e.message);
+            logError(e, 'Unable to read results of FetchCollectionsJob');
         }
 
         if (!valid) {
@@ -214,7 +214,7 @@ const UpdateMtimeJob = new Lang.Class({
                 try {
                     object.update_finish(res);
                 } catch (e) {
-                    log('Unable to run UpdateMtimeJob: ' + e.message);
+                    logError(e, 'Unable to run UpdateMtimeJob');
                 }
 
                 if (this._callback)
@@ -252,7 +252,7 @@ const SetCollectionForSelectionJob = new Lang.Class({
                         try {
                             object.update_finish(res);
                         } catch (e) {
-                            log('Unable to run SetCollectionForSelectionJob: ' + e.message);
+                            logError(e, 'Unable to run SetCollectionForSelectionJob');
                         }
 
                         this._jobCollector();
@@ -294,7 +294,7 @@ const CreateCollectionJob = new Lang.Class({
                 try {
                     variant = object.update_blank_finish(res); // variant is aaa{ss}
                 } catch (e) {
-                    log('Unable to run CreateCollectionJob: ' + e.message);
+                    logError(e, 'Unable to run CreateCollectionJob');
                 }
 
                 variant = variant.get_child_value(0); // variant is now aa{ss}
@@ -798,7 +798,6 @@ const SelectionController = new Lang.Class({
 
     _init: function() {
         this._selection = [];
-        this._selectionMode = false;
 
         Application.documentManager.connect('item-removed',
             Lang.bind(this, this._onDocumentRemoved));
@@ -840,18 +839,6 @@ const SelectionController = new Lang.Class({
             return;
 
         this._isFrozen = freeze;
-    },
-
-    setSelectionMode: function(setting) {
-        if (this._selectionMode == setting)
-            return;
-
-        this._selectionMode = setting;
-        this.emit('selection-mode-changed', this._selectionMode);
-    },
-
-    getSelectionMode: function() {
-        return this._selectionMode;
     }
 });
 Signals.addSignalMethods(SelectionController.prototype);
@@ -869,9 +856,10 @@ const SelectionToolbar = new Lang.Class({
                         'toolbarProperties',
                         'toolbarCollection' ],
 
-    _init: function() {
+    _init: function(overview) {
         this._itemListeners = {};
         this._insideRefresh = false;
+        this._overview = overview;
 
         this.parent();
 
@@ -892,10 +880,9 @@ const SelectionToolbar = new Lang.Class({
         Application.documentManager.connect('active-collection-changed',
             Lang.bind(this, this._updateCollectionsButton));
 
-        Application.selectionController.connect('selection-mode-changed',
-            Lang.bind(this, this._onSelectionModeChanged));
         Application.selectionController.connect('selection-changed',
             Lang.bind(this, this._onSelectionChanged));
+        this._onSelectionChanged();
     },
 
     _updateCollectionsButton: function() {
@@ -907,22 +894,11 @@ const SelectionToolbar = new Lang.Class({
             this._toolbarCollection.show();
     },
 
-    _onSelectionModeChanged: function(controller, mode) {
-        if (mode)
-            this._onSelectionChanged();
-        else
-            this.hide();
-    },
-
     _onSelectionChanged: function() {
-        if (!Application.selectionController.getSelectionMode())
-            return;
-
         let selection = Application.selectionController.getSelection();
         this._setItemListeners(selection);
 
         this._setItemVisibility();
-        this.show();
     },
 
     _setItemListeners: function(selection) {
@@ -946,7 +922,7 @@ const SelectionToolbar = new Lang.Class({
         let hasSelection = (selection.length > 0);
 
         let showTrash = hasSelection;
-        let showPrint = false;
+        let showPrint = hasSelection;
         let showProperties = hasSelection;
         let showOpen = hasSelection;
         let showShare = hasSelection;
@@ -967,21 +943,15 @@ const SelectionToolbar = new Lang.Class({
                     showShare = false;
 
                 showTrash &= doc.canTrash();
+                showPrint &= !doc.collection;
             }));
 
         showOpen = (apps.length > 0);
 
-        if (selection.length == 1) {
-            let doc = Application.documentManager.getItemById(selection[0]);
-            doc.load(null, null, Lang.bind(this,
-                function(doc, docModel, error) {
-                    showPrint = doc.canPrint(docModel);
-                    this._toolbarPrint.set_sensitive(showPrint);
-                }));
-        }
-
-        if (selection.length > 1)
+        if (selection.length > 1) {
+            showPrint = false;
             showProperties = false;
+        }
 
         let openLabel = null;
         if (apps.length == 1) {
@@ -1010,16 +980,15 @@ const SelectionToolbar = new Lang.Class({
             return;
 
         let dialog = new OrganizeCollectionDialog(toplevel);
-        dialog.connect('destroy', Lang.bind(this,
-            function() {
-                Application.selectionController.setSelectionMode(false);
-            }));
+        dialog.connect('destroy', Lang.bind(this, function() {
+            this._overview.getAction('selection-mode').change_state(GLib.Variant.new('b', false));
+        }));
     },
 
     _onToolbarOpen: function(widget) {
-        let selection = Application.selectionController.getSelection();
-        Application.selectionController.setSelectionMode(false);
+        this._overview.getAction('selection-mode').change_state(GLib.Variant.new('b', false));
 
+        let selection = Application.selectionController.getSelection();
         selection.forEach(Lang.bind(this,
             function(urn) {
                 let doc = Application.documentManager.getItemById(urn);
@@ -1045,7 +1014,7 @@ const SelectionToolbar = new Lang.Class({
             }));
 
         let deleteNotification = new Notifications.DeleteNotification(docs);
-        Application.selectionController.setSelectionMode(false);
+        this._overview.getAction('selection-mode').change_state(GLib.Variant.new('b', false));
     },
 
     _onToolbarProperties: function(widget) {
@@ -1055,7 +1024,7 @@ const SelectionToolbar = new Lang.Class({
         dialog.connect('response', Lang.bind(this,
             function(widget, response) {
                 dialog.destroy();
-                Application.selectionController.setSelectionMode(false);
+                this._overview.getAction('selection-mode').change_state(GLib.Variant.new('b', false));
             }));
     },
 
@@ -1065,7 +1034,7 @@ const SelectionToolbar = new Lang.Class({
        dialog.connect('response', Lang.bind(this,
            function(widget, response) {
                dialog.destroy();
-               Application.selectionController.setSelectionMode(false);
+               this._overview.getAction('selection-mode').change_state(GLib.Variant.new('b', false));
            }));
     },
 
@@ -1077,5 +1046,6 @@ const SelectionToolbar = new Lang.Class({
 
         let doc = Application.documentManager.getItemById(selection[0]);
         doc.print(this.get_toplevel());
+        this._overview.getAction('selection-mode').change_state(GLib.Variant.new('b', false));
     },
 });
